# classes/asterisk.pp

class astrad001::asterisk {
	
	file {"/usr/local/bin/bash":
		ensure	=> present,
		source	=> "/bin/bash"
	}

	file {"/usr/local/bin/wget":
		ensure  => present,
		source  => "/usr/bin/wget"
	}

	exec { "asterisk restart":
		command	    => "/etc/init.d/asterisk stop; sh /root/myasterisk/croncheck.sh",
		cwd         => "/root/myasterisk",
		refreshonly => true,
	}

	service { "asterisk":
		ensure => running,
		enable => true,
		require=> Package['asterisk'],
	}

	package { asterisk:		ensure   => installed	}


	exec { "cron restart":
	        path        => "/etc/init.d/",
        	refreshonly => true
	}

	service { "cron":
        	ensure => running,
	        enable => true,
	}


	file {"/etc/asterisk/default-confs":
		ensure	=> directory,
		require => Package["asterisk"],
		notify	=> Exec ["Backup default config files"]
	}


	exec { "Backup default config files":
		refreshonly	=> true,
		command => "mkdir default-confs; mv *.conf default-confs/; mv asterisk.adsi extensions.ael manager.d telcordia-1.adsi default-confs/",
		cwd => "/etc/asterisk",
	}


	include perl
	
	define packagesCPAN {

		perl::installCPAN {    ["YAML", 
					"Config::IniFiles", 
					"Crypt::CBC", 
					"Crypt::DES", 
					"Authen::Radius", 
					"Crypt::Rijndael", 
					"Digest::MD5", 
					"Asterisk::Manager", 
					"List::Util", 
					"Sys::Syslog", 
					"Date::Manip",
					"Time::HiRes"]:
		}
	}
	
	packagesCPAN{"asterisk":
		require	=> Package["asterisk"]
	}

	file { "/root/astrad-src":
		ensure => directory,
		notify => File ["/root/astrad-src/asterisk-perl-1.01.tar.gz"],
	}
	
	# Get the additional dependancy
	file { "/root/astrad-src/asterisk-perl-1.01.tar.gz":
		ensure => file,
		source => "puppet:///$module_name/tar/asterisk-perl-1.01.tar.gz",
		notify =>  Exec ["Extract Asterisk dependancy"],
	}
	
	exec { "Extract Asterisk dependancy":
		refreshonly => true,
		command => "tar xzf asterisk-perl-1.01.tar.gz",
		cwd	=> "/root/astrad-src/",
		creates => "/root/astrad-src/asterisk-perl-1.01",
	}
	# Install dependancy
	exec { "perl Makefile.PL;make all;make install":
		subscribe => Exec[ "Extract Asterisk dependancy"],
		refreshonly => true,
		cwd	=> "/root/astrad-src/asterisk-perl-1.01",
		path    => "/usr/bin:/usr/sbin:/bin:/sbin",
		require	=> PackagesCPAN["asterisk"]
	}


	group{"asterisk":
		ensure	=> present,
		require	=> Package['asterisk']
	}

	# configure asterisk
	file{"/etc/asterisk":
		ensure	=> directory,
		owner   => asterisk,
		group	=> asterisk,
		mode    => 644,
		recurse => true,
		source	=> "puppet:///$module_name/asterisk",
		require => [Package['asterisk'], Service['asterisk'], Group['asterisk']],
		notify  => Exec['asterisk restart']
	}

	# Asterisk conf files
	# using templates	
	
	file{"/etc/asterisk/asterisk.conf":
		ensure	=> file,
		owner   => asterisk,
		group   => asterisk,
		mode	=> 644,
		replace => true,
		content => template("$module_name/asterisk/asterisk.conf-001.erb"),
		require => Package['asterisk'],
		notify  => Exec['asterisk restart']
	}

	file{"/etc/asterisk/billing.conf":
		ensure	=> file,
		owner   => asterisk,
		group   => asterisk,
		mode    => 644,
		replace => true,
		content => template("$module_name/asterisk/billing.conf-001.erb"),
		require => Package['asterisk'],
		notify  => Exec['asterisk restart']
	}

	file{"/etc/asterisk/extensions.conf":
		ensure	=> file,
		owner   => asterisk,
		group   => asterisk,
		mode    => 644,
		replace => true,
		content => template("$module_name/asterisk/extensions.conf-001.erb"),
		require => Package['asterisk'],
		notify  => Exec['asterisk restart']
 	}

	file{"/etc/asterisk/manager.conf":
		ensure	=> file,
		owner   => asterisk,
		group   => asterisk,
		mode    => 644,
		replace => true,
		content => template("$module_name/asterisk/manager.conf-001.erb"),
		require => Package['asterisk'],
		notify	=> Exec['asterisk restart']
	}
	
	# Perl scripts
	# 2 copies of clean register needed	

	file{"/etc/init.d/clean-register.pl":
		owner	=> root,
		group	=> root,
		mode	=> 755,
		content => template("$module_name/perl_scripts/clean-register.pl-001.erb")
	}

	file{"/etc/asterisk/clean-register.pl":
		owner   => root,
		group   => root,
		mode    => 755,
		content => template("$module_name/perl_scripts/clean-register.pl-001.erb"),
		require => Package['asterisk']
	}

	
	file{"/root/myasterisk":
		ensure  => directory,
		owner	=> root,
		group	=> root,
		mode	=> 755,
		recurse	=> true,
		replace	=> true,
		source	=> "puppet:///$module_name/myasterisk"
	}

	file{"/root/myasterisk/ast-resend-lost.pl":
		owner   => root,
		group   => root,
		mode    => 755,
		content => template("$module_name/perl_scripts/ast-resend-lost.pl-002.erb"),
		require => File["/root/myasterisk"]
	}
	
	file{"/root/myasterisk/ast-rad-acc.pl":
		owner	=> root,
		group	=> root,
		mode	=> 755,
		content	=> template("$module_name/perl_scripts/ast-rad-acc.pl-001.erb"),
		require	=> File["/root/myasterisk"]
	}


	# copy agi-bin/agi-rad-auth-2.pl
	file{"/usr/share/asterisk/agi-bin":
		ensure	=> directory,
		owner	=> root,
		group	=> root,
		mode	=> 755,
		recurse	=> true,
		replace	=> true,
		source	=> "puppet:///$module_name/agi-bin",
		require => Package['asterisk']
	}


	file{"/var/log/asterisk/cdr-csv-old":
		ensure	=> directory,
		recurse	=> true,
		replace	=> true,
		require	=> Package['asterisk']
	}



	# Define logrotate
	file{"/etc/logrotate.d/asterisk":
		ensure	=> file,
		source  => "puppet:///$module_name/logrotate.d/asterisk",
		require => [ Package['asterisk'], File['/var/log/asterisk/cdr-csv-old']]
	}


	# Add codecs
	
	file{"/usr/lib/asterisk/modules/codec_g723-ast14-icc-glibc-x86_64-pentium4.so":
		ensure	=> file,
		source	=> "puppet:///$module_name/codec/codec_g723-ast14-icc-glibc-x86_64-pentium4.so",
		notify	=> Exec["asterisk restart"],
		require => Package['asterisk']
	}
		

	file{"/usr/lib/asterisk/modules/codec_g729-ast14-icc-glibc-x86_64-pentium4.so":
		ensure	=> file,
		source	=> "puppet:///$module_name/codec/codec_g729-ast14-icc-glibc-x86_64-pentium4.so",
		notify  => Exec["asterisk restart"],
		require => Package['asterisk']
	}


	# Add sounds	

	file{"/usr/share/asterisk/sounds/FR-invalid-Account.gsm":
		ensure	=> file,
		source	=> "puppet:///$module_name/sounds/FR-invalid-Account.gsm",
		require => Package['asterisk']
	}
		

	file{"/usr/share/asterisk/sounds/FR-login-or-pwd-incorrect.gsm":
		ensure	=> file,
		source	=> "puppet:///$module_name/sounds/FR-login-or-pwd-incorrect.gsm",
		require => Package['asterisk']
	}



	##############################
	# Install asterisk-list script
	
	file{"/root/100105-asterisk-list":
		ensure	=> directory,
	}
	
	# Load KeyAstr
	file{"/root/100105-asterisk-list/KeyAstr":
		ensure	=> file,
		source  => "puppet:///$module_name/KeyAstr",
		replace => true,
		require	=> File["/root/100105-asterisk-list"]
	}

	file{"/root/100105-asterisk-list/Liste-Asterisk.pl":
		ensure	=> file,
		owner	=> root,
		group	=> root,
		mode	=> 755,
		content	=> template("$module_name/perl_scripts/Liste-Asterisk.pl-002.erb"),
		require	=> File["/root/100105-asterisk-list"]
	}




	# 
	file{"/home/sona/100120-registred-sip":
		ensure	=> directory,
		mode	=> 755,
		recurse	=> true,
		replace	=> true,
		source	=> "puppet:///$module_name/100120-registred-sip",
		require	=> User['sona']
	}




	file{"/var/sipenv-$ipaddress":
		ensure => directory,
	}
	
	file{"/var/sipenv-$ipaddress/log":
		ensure	=> directory,
		require	=> File["/var/sipenv-$ipaddress"]
	}
	

	

	define load_mysql_table($db="asterisk", $file, $path = "/root", $module=$module_name, $source){
		
		exec{"mysql -uroot -p$mysql_root_pass < $path/$file":
			refreshonly => true,
			#unless	=> "mysql -uroot -p$mysql_root_pass -e \"show tables from $db like '$name' \" 2> /dev/null",
			require	=> [ Service['mysql'], Exec["Set MySQL root password"], File["$path/$file"]]
		}

		file{"$path/$file":
			ensure	=> file,
			replace	=> true,
			source	=> "puppet:///$module/$source",
			notify  => Exec["mysql -uroot -p$mysql_root_pass < $path/$file"]
		}
		
	}	


	load_mysql_table{"radius_packets":
		file	=> "radius_packets.sql",
		source	=> "mysql/radius_packets.sql"
	}

	load_mysql_table{"radius_packets_Failed":
		file	=> "radius_packets_Failed.sql",
		source	=> "mysql/radius_packets_failed.sql"
	}

	exec{"update-rc.d clean-register.pl defaults":
		refreshonly => true,
		subscribe => File['/etc/init.d/clean-register.pl'],
		require => [Package['rcconf'], File['/etc/init.d/clean-register.pl']],
	}
	

	# define logs
	line{"asterisk.log":
		line => "local0.*                        -/var/log/asterisk.log",
		file => "/etc/rsyslog.conf",
		ensure => present,
	}


	file{"/etc/cron.d/astrad":
		ensure	=> file,
		replace	=> true,
		source	=> "puppet:///$module_name/crontab/astrad",
		notify  => Exec["cron restart"],
	}



	# first update

	file{"/usr/share/asterisk/sounds/":
		ensure  => directory,
		recurse	=> true,
		replace => true,
		source  => "puppet:///$module_name/astrad-prompt",
		require	=> Package["asterisk"],
                notify  => Exec["asterisk restart"]
	}

	# extensions conf in templates

	file{"/etc/asterisk/cdr_custom.conf":
		ensure	=> file,
		owner	=> asterisk,
		group	=> asterisk,
		replace	=> true,
		source  => "puppet:///$module_name/cdr_custom.conf",
		require	=> Package["asterisk"],
		notify	=> Exec["asterisk restart"]
	}

	# agi-rad-auth-2.pl replace old one in agi-bin
	# Liste-Asterisk.pl replace old template
}

